/***************** (c) 2006 BTC Korporacja ***********************************
                     http://www kamami.com
PROJECT  : ADC_LED
BOARD    : ZL1ST7 (with ST7FLITE29 in socket U6)
COMPILER : COSMIC (IDE ST7VD)

******************************************************************************

    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
    BTC KORPORACJA SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT 
    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING 
    FROM USE OF THIS SOFTWARE.

******************************************************************************

DESCRIPTION :
    Example of AD conversion. Result of AD conversion (MSB) is displayed on LEDs.
    LEDs are connected to MCP23S08 (U4) via SPI.
    Jumpers settings on ZL1ST7:
        JP3  - 2-3 - crystal 16MHz
        JP4  - 2-3 - crystal 16MHz
        JP8  - 1-2 - signal from photoresistor (R9)
               2-3 - signal from potentiometer P1
        JP9  - 1-2 - connect analog signal to PB4 (AIN4)

        JP7  - 2-3 - set MCP23S08 address to 0x40
        JP14 - 2-3 - set MCP23S08 address to 0x40
        JP12 - 1-2 - allow SPI transmition via 74xx541 (U6)

    SPI:
        PA5 - CS/SS
        PB3 - MOSI
        PB1 - SCK

******************************************************************************/
                   
#include <io7flit2.h> 
#include "bit.h"

void delay (unsigned int del_time);    // simple delay function
void SPI_Init (void);                  // SPI initialization
void SPI_Tx (char spi_data);           // transmit byte to MCP23S08
unsigned char get_adc(void);

#define CS_pin 5                       // CS - PA5

void main(void)
{
  unsigned int val;
  
  PADDR = 0x20;       // PA5 - CS/SS
  PAOR = 0x20;
  PADR = 0x20;
  PBDDR = 0x0A;       // PB3 - MOSI, PB1 - SCK
  PBOR = 0x0A;
  PBDR = 0x00;
  
  SPI_Init();
   
  while(1)
  {
    SPI_Tx( get_adc() );
    delay(10);
  }                     
}   
/////////////////////////////////////////////////////////////

/*-----------------------------------------------------------------------------
FUNCTION NAME : delay
INPUT/OUTPUT  : del_time
DESCRIPTION   : simple delay function
-----------------------------------------------------------------------------*/ 
void delay (unsigned int del_time) 
{
  unsigned int temp;
                     
  while( del_time-- )
    for(temp=0; temp<5000; temp++)
      ;
} 

                       
/*-----------------------------------------------------------------------------
FUNCTION NAME : SPI_Init
INPUT/OUTPUT  : none
DESCRIPTION   : SPI peripheral initialisation function.
-----------------------------------------------------------------------------*/ 
void SPI_Init (void)
{
  SPICR = 0x10;  // MSTR
  SPISR = 0x03;  // SSM SSI
  SPICR = 0x50;  // enable SPI

  ClrBit(PADR,CS_pin);                 // tie to low CS pin (MCP23S08)
  SPIDR = 0x40;                        // MCP23S08 address
  while(ValBit(SPISR,7)==0) {};        // wait for SPIF bit to go up (=data sent)
  SPIDR;                               // clear the SPIF bit            

  SPIDR = 0x00;                        // MCP23S08 status register address
  while(ValBit(SPISR,7)==0) {};        // wait for SPIF bit to go up (=data sent)
  SPIDR;                               // clear the SPIF bit              

  SPIDR = 0x00;                        // MCP23S08 IO lines as outputs
  while(ValBit(SPISR,7)==0) {};        // wait for SPIF bit to go up (=data sent)
  SPIDR;                               // clear the SPIF bit              

  SetBit(PADR,CS_pin);                 // tie to low CS pin (MPC23S08)
}


/*-----------------------------------------------------------------------------
FUNCTION NAME : SPI_Tx
INPUT/OUTPUT  : spi_data - data byte to transmit.
DESCRIPTION   : transmit byte via SPI to MCP23S08.
-----------------------------------------------------------------------------*/ 
void SPI_Tx (char spi_data)
{        
  ClrBit(PADR,CS_pin);                 // tie to low CS pin (MPC23S08)

  SPIDR = 0x40;                        // MCP23S08 address
  while(ValBit(SPISR,7)==0) {};        // wait for SPIF bit to go up (=data sent)
  SPIDR;                               // clear the SPIF bit            
                                   
  SPIDR = 0x09;                        // MCP23S08 GPIO register
  while(ValBit(SPISR,7)==0) {};        // wait for SPIF bit to go up (=data sent)
  SPIDR;                               // clear the SPIF bit              

  SPIDR = spi_data;    
  while(ValBit(SPISR,7)==0) {};        // wait for SPIF bit to go up (=data sent)
  SPIDR;                               // clear the SPIF bit              
  
  SetBit(PADR,CS_pin);                 // tie to low CS pin (MPC23S08)
}

                       
/*-----------------------------------------------------------------------------
FUNCTION NAME : get_adc
INPUT/OUTPUT  : returns adc result (8 bit)
DESCRIPTION   : AD conversion
-----------------------------------------------------------------------------*/ 
unsigned char get_adc(void)
{
  ADCDRL = 0x08;                       // cklock for ADC - SLOW mode (fcpu/4)
  ADCCSR = ADCCSR | 0x24;              // AIN4 as analog input, ADC on
  while(!(ADCCSR & 0x80))              // wait for end of conversion (EOC)
    ;
  return ADCDRH;                       // return 8 MSB
}
