/***************** (c) 2006 BTC Korporacja ***********************************
                     http://www kamami.com
PROJECT  : LCD
BOARD    : ZL1ST7 (with ST7FLITE29 in socket U6)
COMPILER : COSMIC (IDE ST7VD)

******************************************************************************

    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
    BTC KORPORACJA SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT 
    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING 
    FROM USE OF THIS SOFTWARE.

******************************************************************************

DESCRIPTION :
    Using LCD character display (HD44780 compatible) in 4-bit mode.
    Jumpers settings on ZL1ST7:
        JP3  - 2-3 - crystal 16MHz
        JP4  - 2-3 - crystal 16MHz

    LCD - MCU (RW=GND)
     D4 - PA0
     D5 - PA1
     D6 - PA2
     D7 - PA3
     E  - PA7
     RS - PA4

******************************************************************************/
                   
#include <io7flit2.h> 

void delay (unsigned int del_time);                // simple delay function
void LCDDelay(unsigned int del);                            // delay function for LCD
void LCDText(char *T);                
void LCDTextXY(char x, char y, char *T);
void LCDSendByte(unsigned char data_lcd,unsigned char cmd);
void LCDInit(void);


const LCD_delay = 5;
const unsigned char E  = 0x80;           // PA7
const unsigned char RS = 0x10;           // PA4
const unsigned char dat_lcd = 0x10; 	 // send data to LCD
const unsigned char cmd_lcd = 0x00; 	 // send command to LCD



void main(void)
{     
  PADDR = 0xFF;	                // PORTA: bits 0..7 - outputs
  PAOR = 0xFF;		        // PORTA: push-pull

//  delay(10);   
  LCDInit();                    // LCD initialization
        
  LCDText(" ##  ZL1ST7  ##");   
  delay(10);   
  while(1)
  {
    LCDTextXY(0, 1, " * www.btc.pl * ");
    delay(100);
    LCDTextXY(0, 1, " www.kamami.pl  ");
    delay(100);
  }                     
}   


/*-----------------------------------------------------------------------------
FUNCTION NAME : delay
INPUT/OUTPUT  : del_time
DESCRIPTION   : simple delay function
-----------------------------------------------------------------------------*/ 
void delay (unsigned int del_time) 
{
  unsigned int temp;
                     
  while( del_time-- )
    for(temp=0; temp<5000; temp++)
      ;
} 


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDDelay
INPUT/OUTPUT  : del
DESCRIPTION   : simple delay function for LCD functions
-----------------------------------------------------------------------------*/ 
void LCDDelay(unsigned int del)
{
 for(;del>0;del--)
   ;
} 

/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDGotoXY
INPUT/OUTPUT  : x,y - position on LCD display ( (0,0) = left, up )
DESCRIPTION   : Set actual position to (x,y) 
-----------------------------------------------------------------------------*/ 
void LCDGotoXY(char x, char y)
{
  switch (y)
  {
    case 0:
        x += 0x80;
        break;
    case 1:
        x += 0xC0;
        break;
    case 2:
        x += 0x94;
        break;
    case 3:
        x += 0xD4;
        break;
  }
  LCDSendByte( x , cmd_lcd );
}


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDText
INPUT/OUTPUT  : T - string to be displayed
DESCRIPTION   : Display string T on LCD at actual position 
-----------------------------------------------------------------------------*/ 
void LCDText(char *T)
{
  while (*T)
    LCDSendByte(*T++,dat_lcd);         // display character
}


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDTextXY
INPUT/OUTPUT  : x,y - position on LCD display ( (0,0) = left, up )
                T   - string to be displayed
DESCRIPTION   : Display string T on LCD at position (x,y) 
-----------------------------------------------------------------------------*/ 
void LCDTextXY(char x, char y, char *T)
{
  while (*T)
  {
    LCDGotoXY(x, y);                   // set position
    LCDSendByte(*T,dat_lcd);           // display first character
    x++;                               // next position 
    T++;                               // next character
    if (x > 19)                        // x>19 -> go to next line
    {
      x = 0; 
      y++;
    }
  }  // while
}				   


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDSendByte
INPUT/OUTPUT  : data_lcd - data byte
                cmd      - = dat_lcd (0x10) send data byte to LCD
                           = cmd_lcd (0x00) send command byte to LCD
DESCRIPTION   : LCD initialization
-----------------------------------------------------------------------------*/ 
void LCDSendByte(unsigned char data_lcd,unsigned char cmd)
{
  PADR = ((data_lcd >> 4) & ~(E)) | cmd;    //RS=cmd, E=0 
  LCDDelay( LCD_delay );
  PADR = (data_lcd >> 4) | E | cmd;         //RS=cmd, E=1   
  LCDDelay( LCD_delay );
  PADR = ((data_lcd >> 4) & ~(E)) | cmd;    //RS=cmd, E=0 
  LCDDelay( LCD_delay );
    
  PADR = (data_lcd & ~(E)) | cmd;           //RS=cmd, E=0
  LCDDelay( LCD_delay );
  PADR = (data_lcd) | E | cmd;              //RS=cmd, E=1
  LCDDelay( LCD_delay );
  PADR = (data_lcd & ~(E)) | cmd;           //RS=cmd, E=0  
  LCDDelay( LCD_delay ); 
} 


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDInit
INPUT/OUTPUT  : none
DESCRIPTION   : LCD initialization
-----------------------------------------------------------------------------*/ 
void LCDInit(void)
{
  LCDDelay(10000);
  
  PADR = 0x03;         //D5=1, D4=1
  PADR = 0x83;         //E=1
  PADR = 0x03;         //E=0
  LCDDelay( 1000 );
  
  PADR = 0x03;         //D5=1, D4=1
  PADR = 0x83;         //E=1
  PADR = 0x03;         //E=0
  LCDDelay( 1000 );
  
  PADR = 0x03;         //D5=1, D4=1
  PADR = 0x83;         //E=1
  PADR = 0x03;         //E=0
  LCDDelay( 1000 );
  
  PADR = 0x02;         //D5=1, D4=0
  PADR = 0x82;         //E=1
  PADR = 0x02;         //E=0
  LCDDelay(1000);

  LCDSendByte( 0x28, cmd_lcd);         // 2 lines, 4 bits 
  LCDDelay(1000);                                          
  LCDSendByte( 0x08, cmd_lcd);         // LCD off                           
  LCDDelay(1000);                
  LCDSendByte( 0x06, cmd_lcd);         // cursor inc
  LCDDelay(1000);
  LCDSendByte( 0x01, cmd_lcd);         // clear LCD, cursor home
  LCDDelay(1000);
  LCDSendByte( 0x0c, cmd_lcd);         // LCD on
  LCDDelay(5000);
}   
